' The MP3Decoder class encapsulates the details of decoding an MP3 audio frame
Public Class MP3Decoder

#Region " Private Variables "
    ' The AudioOutputBuffer instance that will receive the decoded PCM samples.
    Private m_OutputBuffer As AudioOutputBuffer
    ' Synthesis filter for the left channel
    Private m_filter1 As SynthesisFilter
    ' Synthesis filter for the right channel
    Private m_filter2 As SynthesisFilter
    ' The decoder used to decode layer 3 frames
    Private m_l3decoder As Layer3Decoder
    Private m_initialized As Boolean
#End Region

#Region " Properties "
    ' Changes the output buffer. This will take effect the next time
    ' decodeFrame() is called.
    Public WriteOnly Property OutputBuffer() As AudioOutputBuffer
        Set(ByVal Value As AudioOutputBuffer)
            m_OutputBuffer = Value
        End Set
    End Property
#End Region

#Region " Functions "

    ' Decodes one frame from an MPEG audio BitStream
    ' header    :	The header describing the frame to decode
    ' BitStream :	The bistream that provides the bits for te body of the frame.
    ' returns   :   A SampleBuffer containing the decoded samples.
    Public Function DecodeFrame(ByVal m_FrameHeader As FrameHeader, ByVal m_Bitstream As Bitstream) As AudioOutputBuffer
        If Not m_initialized Then
            Initialize(m_FrameHeader)
        End If
        m_OutputBuffer.ClearBuffer()

        If m_l3decoder Is Nothing Then
            m_l3decoder = New Layer3Decoder(m_Bitstream, m_FrameHeader, m_filter1, m_filter2, m_OutputBuffer, OutputChannelsEnum.BOTH_CHANNELS)
        End If
        m_l3decoder.DecodeFrame()
        m_OutputBuffer.WriteBuffer(1)
        Return m_OutputBuffer

    End Function

    Private Sub Initialize(ByVal m_FrameHeader As FrameHeader)
        Dim scalefactor As Single = 32700.0F

        Dim mode As Integer = m_FrameHeader.Mode()

        Dim channels As Integer
        If mode = FrameHeader.SINGLE_CHANNEL Then
            channels = 1
        Else
            channels = 2
        End If
        'If m_OutputBuffer Is Nothing Then
        '    m_OutputBuffer = New SampleBuffer(channels)
        'End If
        m_filter1 = New SynthesisFilter(0, scalefactor)

        If channels = 2 Then
            m_filter2 = New SynthesisFilter(1, scalefactor)
        End If
        m_initialized = True
    End Sub

#End Region

End Class